#ifndef ___HGL___counter___
#define ___HGL___counter___

/*
 CCounter
 基本カウンタークラス。
 基本的なcounterであり、使用方法は
 CCounter hoge(1,0,100);
 とすると、初期値が0、最大値が100、一回のインクリメントにつき
 1ずつ増加する、というカウンターになる。
 この時、カウントが最大値を超えたら、初期値に戻る
 CCounter hoge(2,0,100,true)
 とすると、超えた分の値を持ち越す
*/
class CCounter {
public:

	CCounter(int nMargin,int nInit,int nMax,int nMin, bool b = false) {
		m_nMax = nMax;
		m_nInit = nInit;
		m_nMargin = nMargin;
		m_nNowCount = nInit;
		m_nMin = nMin;
		m_bCarry = b;
	}

	CCounter() {
		m_nMax = 0;
		m_nInit = 0;
		m_nMargin = 0;
		m_nNowCount = 0;
		m_nMin = 0;
		m_bCarry = false;
	}

	//オペレーターオーバーロード
	//インクリメント、デクリメント、-=、+=、==、<><=>=
	//などを全て判定できるようにする。
	CCounter& operator++() {inc();return *this;}
	CCounter& operator++(int n) {inc();return *this;}
	CCounter& operator--() {dec();return *this;}
	CCounter& operator--(int n) {dec();return *this;}

	CCounter& operator+=(int n) {inc(n);return *this;}
	CCounter& operator-=(int n) {dec(n);return *this;}

	bool operator!=(int n) {return get() != n?true:false;}
	bool operator==(int n) {return get() == n?true:false;}
	bool operator>=(int n) {return get() >= n?true:false;}
	bool operator<=(int n) {return get() <= n?true:false;}
	bool operator>(int n) {return get() > n?true:false;}
	bool operator<(int n) {return get() < n?true:false;}
	bool operator==(const CCounter& n) {return get() == n.get()?true:false;}
	bool operator>=(const CCounter& n) {return get() >= n.get()?true:false;}
	bool operator<=(const CCounter& n) {return get() <= n.get()?true:false;}
	bool operator>(const CCounter& n) {return get() > n.get()?true:false;}
	bool operator<(const CCounter& n) {return get() < n.get()?true:false;}

	/**
	 * 現在の値を取得する
	 * @access public
	 * @return int
	 */
	int get() const {return m_nNowCount;}

	/**
	 * 状態を初期化する。初期化される値は、初期値として設定した値である。
	 * @access public
	 * @return void
	 */
	void reset() {m_nNowCount = m_nInit;}

	/**
	 * マージンを設定する
	 * @param int nMargin マージン
	 * @access public
	 * @return void
	 */
	void setMargin(int nMargin) {m_nMargin = nMargin;}
	
	/**
	 * マージンを取得する
	 * @access public
	 * @return int
	 */
	int getMargin() const {return m_nMargin;}

	/**
	 * 初期値を設定する
	 * @param int nInit 初期値
	 * @access public
	 * @return void
	 */
	void setInit(int nInit) {m_nInit = nInit;}
	
	/**
	 * 初期値を取得する
	 * @access public
	 * @return int
	 */
	int getInit() const {return m_nInit;}

	/**
	 * 最大値を設定する
	 * @param int nMax 最大値
	 * @access public
	 * @return void
	 */
	void setMax(int nMax) {m_nMax = nMax;}
	
	/**
	 * 最大値を取得する
	 * @access public
	 * @return int
	 */
	int getMax() const {return m_nMax;}

	int getMin() const {return m_nMin;}
	void setMin(int min) {m_nMin = min;}

private:

	/**
	 * @brief カウンタのインクリメントを行う。このカウンタでは、
	 * 単純なインクリメントによる、内部マージンの加算を行う。
	 */
	void inc() {
		//内部マージンの加算
		m_nNowCount += m_nMargin;

		if (!m_bCarry) {
			//加算するマージンによって、どちらと判定するかを決定する。
			if (m_nMargin > 0) {
				if (m_nNowCount > m_nMax) {
					m_nNowCount = m_nMax;
				}
			} else {
				if (m_nNowCount < m_nMin) {
					m_nNowCount = m_nMin;
				}
			}
		} else {
			//キャリーオーバーするのは、あくまで最大値のみ。
			if (m_nNowCount > m_nMax) {
				//キャリーオーバーは、最大値と最小値それぞれの絶対値の差分を引く。
				m_nNowCount -= (abs(m_nMax) - abs(m_nMin));
			} else if (m_nNowCount < m_nMin) {
				m_nNowCount += (abs(m_nMax) - abs(m_nMin));
			}
		}
	}

	/**
	 * カウンタのインクリメントを行う
	 * @param int nInc 指定された数だけインクリメントする。
	 * @access protected
	 * @return void
	 */
	void inc(int nInc) {
		
		m_nNowCount += nInc;

		if (!m_bCarry) {
			if (nInc > 0) {
				if (m_nNowCount > m_nMax) {
					m_nNowCount = m_nMax;
				}
			} else {
				if (m_nNowCount < m_nMin) {
					m_nNowCount = m_nMin;
				}
			}
		} else {
			//キャリーオーバーするのは、あくまで最大値のみ。
			if (m_nNowCount > m_nMax) {
				//キャリーオーバーは、最大値と最小値それぞれの絶対値の差分を引く。
				m_nNowCount -= (abs(m_nMax) - abs(m_nMin));
			} else if (m_nNowCount < m_nMin) {
				m_nNowCount += (abs(m_nMax) - abs(m_nMin));
			}
		}
	}

	/**
	 * @brief 単純なインクリメントによる、内部マージンの減算。
	 */
	void dec() {
		m_nNowCount -= m_nMargin;

		if (!m_bCarry) {
			//減算するマージンによって、どちらと判定するかを決定する。
			if (m_nMargin < 0) {
				if (m_nNowCount > m_nMax) {
					m_nNowCount = m_nMax;
				}
			} else {
				if (m_nNowCount < m_nMin) {
					m_nNowCount = m_nMin;
				}
			}
		} else {
			//キャリーオーバーするのは、あくまで最大値のみ。
			if (m_nNowCount > m_nMax) {
				//キャリーオーバーは、最大値と最小値それぞれの絶対値の差分を引く。
				m_nNowCount -= (abs(m_nMax) - abs(m_nMin));
			} else if (m_nNowCount < m_nMin) {
				m_nNowCount += (abs(m_nMax) - abs(m_nMin));
			}
		}
	}

	/**
	 * カウンタのデクリメントを行う
	 * @param int nDec デフォルトは-1であり、-1である場合は設定値が使われる
	 * @access protected
	 * @return void
	 */
	void dec(int nDec) {
		m_nNowCount -= nDec;

		if (!m_bCarry) {
			if (nDec < 0) {
				if (m_nNowCount > m_nMax) {
					m_nNowCount = m_nMax;
				}
			} else {
				if (m_nNowCount < m_nMin) {
					m_nNowCount = m_nMin;
				}
			}
		} else {
			//キャリーオーバーするのは、あくまで最大値のみ。
			if (m_nNowCount > m_nMax) {
				//キャリーオーバーは、最大値と最小値それぞれの絶対値の差分を引く。
				m_nNowCount -= (abs(m_nMax) - abs(m_nMin));
			} else if (m_nNowCount < m_nMin) {
				m_nNowCount += (abs(m_nMax) - abs(m_nMin));
			}
		}
	}

private:

	int m_nNowCount;	//現在カウント
	int m_nInit;		//初期値
	int m_nMargin;		//一回の増減値
	int m_nMax;			//最大値
	int m_nMin;			//最低値
	bool m_bCarry;		//持ち越しを行うかどうか
};

/**
 CFractionalCounter
 分数関数を用いたカウンター。
 具体的な使用方法は、次のような用途に。

 //0から22までを、三回のインクリメントで。
 //最高値に達したら、0にもどらない
 CFractionalCounter counter(0,22,3);
 counter++;

 //0から22まで、三回のインクリメントで。
 //最高値に達した場合、0に戻ってもう一度行う。
 CFractionalCounter counter(0,22,3,true);

 //最大値よりも最小値が小さい場合でも、意味は同じ
 CFractionalCounter counter(23,0,3);
 counter++;			
*/
class CFractionalCounter {
public:

	CFractionalCounter(int nInit,int nThreshold,int nCount,bool bLoop = false) {
		set(nInit,nThreshold,nCount,bLoop);
	}
	CFractionalCounter() {
		set(0,0,0,false);
	}
	
	//基本的にインクリメントのみ。
	//比較などしようがないかもしれないが、intとの比較くらいは出来るように。
	CFractionalCounter& operator++() {inc();return *this;}
	CFractionalCounter& operator++(int n) {inc();return *this;}
	
	//他のカウンターと使用方法が違うので、intとの比較だけで十分だろう。
	bool operator==(int n) {return (m_dwNowCount >> 16) == n?true:false;}
	bool operator>=(int n) {return (m_dwNowCount >> 16) >= n?true:false;}
	bool operator<=(int n) {return (m_dwNowCount >> 16) <= n?true:false;}
	bool operator>(int n) {return (m_dwNowCount >> 16) > n?true:false;}
	bool operator<(int n) {return (m_dwNowCount >> 16) < n?true:false;}

	/**
	 * 現在の値を取得する
	 * @access public
	 * @return int
	 */
	int get() const {return (m_dwNowCount >> 16);}

	/**
	 * 初期状態に戻す
	 * @access public
	 * @return void
	 */
	void reset() {
		m_dwNowCount = m_nInit << 16;
		m_nNowNum = 0;
	}

	/**
	 * 変数の設定、コピーで使用する。
	 * @param int nInit 初期値
	 * @param int nMax 最大値
	 * @param int nCount 初期値から最大値までの回数
	 * @param bool bLoop ループするかどうか
	 * @access public
	 * @return void
	 */
	void set(int nInit,int nThreshold,int nCount,bool bLoop = false) {
		m_nInit = nInit;
		m_nThreshold = nThreshold;
		m_dwNowCount = nInit << 16;
		m_nNum = nCount;
		m_bLoop = bLoop;
		m_nNowNum = 0;
		calc();
	}

private:

	/**
	 * 計算を行う
	 * @access protected
	 * @return void
	 */
	void calc() {
		//計算を行う。ここで行う計算とは、与えられた値を元に、
		//一回ずつの増分、減分を取得することである。
		unsigned long tmp = abs(m_nThreshold - m_nInit) << 16;
		(m_nThreshold - m_nInit) <= 0?m_bReduce = true:m_bReduce = false;

		//16ビットまでしか計算できない。が、十分だろ。
		m_dwMargin = tmp / m_nNum;
	}

	/**
	 * 一回分の増減を行う。
	 * @access protected
	 * @return void
	 */
	void inc() {
		//一回分の加算・減算を行う。
		//誤差が出たりするのは仕方ないと思う。
		
		if (m_nNowNum >= m_nNum) {
			if (m_bLoop) {
				m_dwNowCount = m_nInit << 16;
				m_nNowNum = 0;
			} else {
				m_dwNowCount = m_nThreshold << 16;
				m_nNowNum = m_nNum;
			}
		} else {
			m_nNowNum++;
			if (m_bReduce) {
				m_dwNowCount -= m_dwMargin;
			} else {
				m_dwNowCount += m_dwMargin;
			}
		}
	}	

private:

	int m_nInit;
	int m_nThreshold;
	unsigned long m_dwNowCount;
	int m_nNum;
	int m_nNowNum;
	bool m_bLoop;

	bool m_bReduce;		//マイナスかどうか

	unsigned long m_dwMargin;		//一回のマージン

};
#endif