#include "stdafx.h"
#include "file.h"

CFile::CFile()
{
	m_lpFile = NULL;
	m_bOpenFlg = false;
}

CFile::CFile(const std::string& fname,FOPEN_MODE mode)
{
	m_lpFile = NULL;
	m_bOpenFlg = false;
	open(fname,mode);
}

CFile::~CFile()
{
	close();
}

int CFile::open(const std::string& fname,FOPEN_MODE mode)
{
	if (m_lpFile != NULL) {
		close();
	}

	int result = 0;
	switch (mode) {
		case READ:
			result = innerOpen(fname,"r");
			break;
		case WRITE:
			result = innerOpen(fname,"w");
			break;
		case ADD:
			result = innerOpen(fname,"a");
			break;
		case READ_B:
			result = innerOpen(fname,"rb");
			break;
		case WRITE_B:
			result = innerOpen(fname,"wb");
			break;
		case ADD_B:
			result = innerOpen(fname,"ab");
			break;
		case RW:
			result = innerOpen(fname,"r+");
			break;
		case RW_B:
			result = innerOpen(fname,"rb+");
			break;
	}

	return result;
}

int CFile::innerOpen(const std::string& fname,const std::string& mode)
{
	m_lpFile = fopen(fname.c_str(),mode.c_str());
	if (m_lpFile == NULL) {
		return 2;
	}

	m_bOpenFlg = true;
	
	//ファイルサイズの取得
	if (mode.compare("r") == 0 || mode.compare("rb") == 0) {
		end();
		m_lFileSize = ftell(m_lpFile);
		begin();
	}

	return 0;
}

int CFile::close()
{
	if (m_lpFile != NULL) {
		fclose(m_lpFile);
	}
	m_lpFile = NULL;
	return 0;
}

int CFile::begin()
{
	if (!isOpen()) {
		return 1;
	}

	return fseek(m_lpFile,0,0);
}

int CFile::end()
{
	if (!isOpen()) {
		return 1;
	}
	
	return fseek(m_lpFile,0,2);
}

int CFile::read(unsigned char* buf,unsigned long readsize)
{
	if (!isOpen()) {
		return -1;
	}

	if (readsize == 0) {
		return -2;
	}

	long result = fread(buf,1,readsize,m_lpFile);
	if (result < readsize) {
		return readsize - result;
	}
	return result;
}

int CFile::read(std::vector<unsigned char>& buf,unsigned long readsize)
{
	if (!isOpen()) {
		return 1;
	}

	if (readsize == 0) {
		return 2;
	}

	if (buf.size() < readsize) {
		buf.resize(readsize);
	}

	long result = fread(&(buf[0]),1,readsize,m_lpFile);
	if (result < readsize) {
		buf.resize(result);
		return result;
	}
	return result;
}

int CFile::readLine(std::string& buf)
{
	//一行を読み出す
	if (!isOpen()) {
		return 1;
	}

	if (!buf.empty()) {
		buf.assign("");
	}

	//改行まで読み出す
	while (!isEOF()) {
		unsigned char c;
		if (read(&c,1) != 1) {
			return 2;
		}

		//めんどくさいので、Windows標準の改行だけチェック
		if (c == '\r') {
			unsigned char tmp;
			if (read(&tmp,1) != 1) {
				return 3;
			}
			if (tmp == '\n') {
				//これが出たら終了
				break;
			} else {
				//おかしいので削除してやる
				break;
			}
		} else if (c == '\n') {
			return 0;
		} else {
			buf += c;
		}
	}

	return 0;
}

int CFile::write(const std::string& fname)
{
	if (!isOpen()) {
		return 1;
	}

	int result = fwrite((void*)fname.c_str(),1,fname.size(),m_lpFile);

	if (result < fname.size()) {
		return 2;
	}

	return 0;
}

int CFile::write(void *lpmem,unsigned long size)
{
	if (!isOpen()) {
		return 1;
	}

	long result = fwrite(lpmem,1,size,m_lpFile);

	if (result < size) {
		
		return 2;
	}

	return 0;
}

int CFile::load(const std::string& fname,unsigned char* buf,unsigned long readsize,
				FOPEN_MODE mode)
{
	if (open(fname,mode)) {
		return 1;
	}

	if (read(buf,readsize)) {
		return 2;
	}

	return 0;
}

int CFile::seek(unsigned long seeksize)
{
	if (!isOpen()) {
		return 1;
	}

	int res = fseek(m_lpFile,seeksize,1);
	if (res != 0) {
		return 2;
	}

	return 0;
}

bool CFile::isEOF()
{
	if (!isOpen()) {
		return true;
	}

	if (feof(m_lpFile)) {
		return true;
	}
	return false;
}
