#ifndef _UTAKATA_SRC_CLASS_H_
#define _UTAKATA_SRC_CLASS_H_

#include <string>

namespace utakata {

namespace interpreter {
class IClass {
  // インタープリタ内におけるすべてのオブジェクトのメタクラス
  // の基底となる。
  // 実際には、これを継承したテンプレートが利用されるようになる。
 public:

  IClass() {}
  virtual ~IClass() {}

  // Classに関連付けられたオブジェクトを生成する。
  // virtual interpreter::IObject* create() = 0;

  // Classに関連付けられたオブジェクトの型を返す。
  virtual std::string class_name() const = 0;

  bool operator==(const IClass& rh) {
    return class_name() == rh.class_name();
  }
  bool operator!=(const IClass& rh) {
    return !(*this == rh);
  }
};

template<class T>
class Class : public IClass {
  // IClassのテンプレート継承したクラス。実際にはこちらを利用
  // する。各IObjectは、各自このクラスを生成時に作成して返す
  // ことになる。また、このクラスはそれぞれの型に対する
  // TypeHashを作成する際に利用される。同一のクラスであれば、
  // 必ず同一の文字列が返されることが保証されるため、
  // TypeHashの仕組みが簡単になり、結果として処理をシンプルに
  // することができる。
 public:
  Class() {}
  virtual ~Class() {}

  virtual std::string class_name() const {
    return "";
  }
};
};
};

#endif /* _UTAKATA_SRC_CLASS_H_ */
