#ifndef _DEBUGUTIL_H_
#define _DEBUGUTIL_H_

#include <algorithm>
#include <iostream>                     // NOLINT
#include <string>
#include <functional>

namespace utakata {

namespace debugutil {

///////////////////////////////////////////////////////////////
//  utakata内部で利用される、デバッグ用ユーティリティ。      //
//  それぞれ、デバッグの際にのみ有効になるようになっており、 //
//  DEBUG_がdefineされている場合は単なる空関数となる。       //
///////////////////////////////////////////////////////////////

template<class T>
struct echo_iostream : std::unary_function<T, void> {
  explicit echo_iostream(std::string delim) : delim_(delim) {}

  void operator()(T t) {
    std::cout << t << delim_;
  }
  std::string delim_;
};

template<class S>
echo_iostream<typename S::value_type>
get_echo_iostream(S type, std::string delim) {
  return echo_iostream<typename S::value_type>(delim);
}

template<class S>
echo_iostream<typename S::value_type> get_echo_iostream(S type) {
  return echo_iostream<typename S::value_type>("");
}

template<class T>
void echo_nl(T s) {
  // 与えたデータを改行有で表示する。
  echo_iostream<T>("\n")(s);
}

template<class T>
void echo(T s) {
  // 与えたデータをデリミタ無しで表示する。
  echo_iostream<T>("")(s);
}

template<class T>
void echo(T s, std::string d) {
  echo_iostream<T> t(d);
  t(s);
}

template<class T>
void echoc_nl(const T& container_) {
  // 渡されたコンテナの各要素をコンソールに出力する。引数が設定されている
  // 場合には、改行無しで表示する。
  std::string d = "\n";
  std::for_each(container_.begin(), container_.end(),
                get_echo_iostream(container_, d));
}

template<class T>
void echoc_nl(T begin, T end) {
  // 渡されたiteratorのbeginからendまでを標準出力に
  // 出力する。
  std::string d = "\n";
  std::for_each(begin, end,
                get_echo_iostream(begin, d));
}


template<class T>
void echoc(const T& container_) {
  // 渡されたコンテナの各要素をコンソールに出力する。引数が設定されている
  // 場合には、改行無しで表示する。
  std::for_each(container_.begin(), container_.end(),
                get_echo_iostream(container_));
}


template<class T>
void echoc(const T& container_, std::string delim) {
  // containerの各要素をdelimで区切って表示する。
  // delimを指定しない場合、一切区切らずに表示される。
  std::for_each(container_.begin(), container_.end(),
                get_echo_iostream(container_, delim));
}

template<class T>
void echoc(T begin, T end, std::string delim) {
  // 渡されたiteratorのbeginからendまでを標準出力に
  // 出力する。

  std::for_each(begin, end,
                get_echo_iostream(begin, delim));
}
};  //  end of namespace debugutil
};  //  end of namespace utakata

#endif /* _DEBUGUTIL_H_ */
