#ifndef ___HGL___FUNCTION_CALLBACK___
#define ___HGL___FUNCTION_CALLBACK___

#include "src/common/disable_copy.h"

namespace fc {

/**
   function_callback
   関数コールバック基底クラス
   全てのコールバック関数クラスは、このクラスから派生する。
   smart_ptrの使用が前提である。
*/
class FunctionCallback : private utility::DisableCopyInterface {
 public:

  virtual ~FunctionCallback() {}

  //  これを必ずオーバーロードすること。
  virtual void Run() = 0;
};

/*
  準備するコールバック関数クラスの種類は、
  返り値あり、返り値無し --- 2種
  メンバ関数、非メンバ関数 --- 2種
  引数0〜5個               --- 6種
  の24個定義する。
  それぞれ、FunctionCallback[返り値有無][Global/Member][引数の個数]
  というフォーマットのクラス名となっている。
*/
template<class T> class FunctionCallbackVoidGlobal0;
class FunctionCallbackVoid0;
template<class T> class FunctionCallbackVoidGlobal1;
template<class T1, class T2> class FunctionCallbackVoid2;
template<class T1, class T2, class T3> class FunctionCallbackVoid3;
template<class T1, class T2, class T3, class T4> class FunctionCallbackVoid4;
template<class T1, class T2, class T3, class T4, class T5>
class FunctionCallbackVoid5;

template<class Obj> class FunctionCallbackVoidMember0;
template<class Obj, class T> class FunctionCallbackVoidMember1;
template<class Obj, class T1, class T2> class FunctionCallbackVoidMember2;
template<class Obj, class T1, class T2, class T3>
class FunctionCallbackVoidMember3;
template<class Obj, class T1, class T2, class T3, class T4>
class FunctionCallbackVoidMember4;
template<class Obj, class T1, class T2, class T3, class T4, class T5>
class FunctionCallbackVoidMember5;

/**
   FunctionCallback_v
   返り値無しのコールバック関数クラス。
   staticなcreate関数によって、それぞれを作成する。
   また、これから作成されるクラスも、全てFunctionCallback派生クラスである。
*/
class FunctionCallbackVoid : public FunctionCallback {
 public:

  virtual ~FunctionCallbackVoid() {}

  //----------------------------------------------------------------------
  // 各関数コールバッククラス生成関数群。
  // ここでの関数群は、全て同じ名前のオーバーロードによって行われる。
  //----------------------------------------------------------------------

  //  引数無し、返り値無しの静的関数のみのコールバック関数クラス作成
  template<class T>
  static FunctionCallbackVoid* create(void (T::*f)()) {
    return new FunctionCallbackVoidg0<T>(f);
  }

  //  引数あり、静的関数のみ
  template<class Arg1>
  static FunctionCallbackVoid* create(void (*f)(Arg1), Arg1 a1) {
    return new FunctionCallbackVoid1<Arg1>(f, a1);
  }

  template<class Arg1, class Arg2>
  static FunctionCallbackVoid* create(void (*f)(Arg1, Arg2),
                                     Arg1 a1, Arg2 a2) {
    return new FunctionCallbackVoid2<Arg1, Arg2>(f, a1, a2);
  }

  template<class Arg1, class Arg2, class Arg3>
  static FunctionCallbackVoid* create(void (*f)(Arg1, Arg2, Arg3),
                                     Arg1 a1, Arg2 a2, Arg3 a3) {
    return new FunctionCallbackVoid3<Arg1, Arg2, Arg3>(f, a1, a2, a3);
  }

  template<class Arg1, class Arg2, class Arg3, class Arg4>
  static FunctionCallbackVoid* create(void (*f)(Arg1, Arg2, Arg3, Arg4),
                                     Arg1 a1, Arg2 a2, Arg3 a3, Arg4 a4) {
    return new FunctionCallbackVoid4<Arg1, Arg2, Arg3, Arg4>
        (f, a1, a2, a3, a4);
  }

  template<class Arg1, class Arg2, class Arg3, class Arg4, class Arg5>
  static FunctionCallbackVoid*
  create(void (*f)(Arg1, Arg2, Arg3, Arg4, Arg5), Arg1 a1, Arg2 a2,
         Arg3 a3, Arg4 a4, Arg5 a5) {
    return new FunctionCallbackVoid5<Arg1, Arg2, Arg3, Arg4, Arg5>
        (f, a1, a2, a3, a4, a5);
  }

  //  引数無し、メンバ関数
  template<class obj>
  static FunctionCallbackVoid* create(void (obj::*f)(), obj* p) {
    return new FunctionCallbackVoidMember0<obj>(f, p);
  }

  template<class obj, class Arg1>
  static FunctionCallbackVoid* create(void (obj::*f)(Arg1), obj* p,
                                     Arg1 a1) {
    return new FunctionCallbackVoidMember1<obj, Arg1>(f, p, a1);
  }

  template<class obj, class Arg1, class Arg2>
  static FunctionCallbackVoid* create(void (obj::*f)(Arg1, Arg2), obj* p,
                                     Arg1 a1, Arg2 a2) {
    return new FunctionCallbackVoidMember2<obj, Arg1, Arg2>(f, p, a1, a2);
  }

  template<class obj, class Arg1, class Arg2, class Arg3>
  static FunctionCallbackVoid* create(void (obj::*f)(Arg1, Arg2, Arg3),
                                     obj* p, Arg1 a1, Arg2 a2, Arg3 a3) {
    return new FunctionCallbackVoidMember3<obj, Arg1, Arg2, Arg3>
        (f, p, a1, a2, a3);
  }

  template<class obj, class Arg1, class Arg2, class Arg3, class Arg4>
  static FunctionCallbackVoid*
  create(void (obj::*f)(Arg1, Arg2, Arg3, Arg4), obj* p,
         Arg1 a1, Arg2 a2, Arg3 a3, Arg4 a4) {
    return new FunctionCallbackVoidMember4<obj, Arg1, Arg2, Arg3, Arg4>
        (f, p, a1, a2, a3, a4);
  }

  template<class obj, class Arg1, class Arg2, class Arg3,
           class Arg4, class Arg5>
  static FunctionCallbackVoid*
  create(void (obj::*f)(Arg1, Arg2, Arg3, Arg4, Arg5), obj* p,
         Arg1 a1, Arg2 a2, Arg3 a3, Arg4 a4, Arg5 a5) {
    return new FunctionCallbackVoidMember5<obj, Arg1, Arg2, Arg3, Arg4, Arg5>
        (f, p, a1, a2, a3, a4, a5);
  }

  virtual void Run() {}
};

//==============================================================================
//  返り値ありの関数コールバッククラス
//==============================================================================

template<class Result, class T> class FunctionCallbackResultGlobal0;

template<class Result>class FunctionCallbackResult0;

template<class Result, class T> class FunctionCallbackResult1;

template<class Result, class T1, class T2> class FunctionCallbackResult2;

template<class Result, class T1, class T2, class T3> class FunctionCallbackResult3;

template<class Result, class T1, class T2, class T3, class T4>
class FunctionCallbackResult4;

template<class Result, class T1, class T2, class T3, class T4, class T5>
class FunctionCallbackResult5;

template<class Result, class OBJ> class FunctionCallbackResultMember0;

template<class Result, class OBJ, class T> class FunctionCallbackResultMember1;

template<class Result, class OBJ, class T1, class T2>
class FunctionCallbackResultMember2;

template<class Result, class OBJ, class T1, class T2, class T3>
class FunctionCallbackResultMember3;

template<class Result, class OBJ, class T1, class T2, class T3, class T4>
class FunctionCallbackResultMember4;

template<class Result, class OBJ, class T1, class T2,
         class T3, class T4, class T5>
class FunctionCallbackResultMember5;

/**
   FunctionCallback_r
   返り値ありのコールバック関数クラスの元締め
   getResultで引数を取得する
*/
template<class Result>
class FunctionCallbackResult : public FunctionCallback {
 public:

  FunctionCallbackResult() : mResultesult() {}
  virtual ~FunctionCallbackResult() {}

  //----------------------------------------------------------------------------
  //  返り値有りの関数コールバックの定義
  //  返り値はそれぞれ自由に設定することができる。
  //----------------------------------------------------------------------------
  inline FunctionCallbackResult<Result>* create(Result (*f)(void)) {
    return new FunctionCallbackResult0<Result>(f);
  }


  template<class T>
  static FunctionCallbackResult<Result>* create(Result (T::*f)()) {
    return new FunctionCallbackResultGlobal0<Result, T>(f);
  }

  //  非メンバ関数
  template<class Arg1>
  static FunctionCallbackResult<Result>* create(Result (*f)(Arg1), Arg1 a1) {
    return new FunctionCallbackResult1<Result, Arg1>(f, a1);
  }

  template<class Arg1, class Arg2>
  static FunctionCallbackResult<Result>* create(Result (*f)(Arg1, Arg2),
                                             Arg1 a1, Arg2 a2) {
    return new FunctionCallbackResult2<Result, Arg1, Arg2>(f, a1, a2);
  }

  template<class Arg1, class Arg2, class Arg3>
  static FunctionCallbackResult<Result>*
  create(Result (*f)(Arg1, Arg2, Arg3), Arg1 a1, Arg2 a2, Arg3 a3) {
    return new FunctionCallbackResult3<Result, Arg1, Arg2, Arg3>(f, a1, a2, a3);
  }

  template<class Arg1, class Arg2, class Arg3, class Arg4>
  static FunctionCallbackResult<Result>*
  create(Result (*f)(Arg1, Arg2, Arg3, Arg4),
         Arg1 a1, Arg2 a2, Arg3 a3, Arg4 a4) {
    return new FunctionCallbackResult4<Result, Arg1, Arg2, Arg3, Arg4>
        (f, a1, a2, a3, a4);
  }

  template<class Arg1, class Arg2, class Arg3, class Arg4, class Arg5>
  static FunctionCallbackResult<Result>*
  create(Result (*f)(Arg1, Arg2, Arg3, Arg4),
         Arg1 a1, Arg2 a2, Arg3 a3, Arg4 a4, Arg5 a5) {
    return new FunctionCallbackResult5<Result, Arg1, Arg2, Arg3, Arg4, Arg5>
        (f, a1, a2, a3, a4, a5);
  }

  //  メンバ関数
  template<class T>
  static FunctionCallbackResult<Result>* create(Result (T::*f)(), T* p) {
    return new FunctionCallbackResultMember0<Result, T>(f, p);
  }

  template<class T, class Arg1>
  static FunctionCallbackResult<Result>* create(Result (T::*f)(Arg1), T* p,
                                             Arg1 a1) {
    return new FunctionCallbackResultMember1<Result, T, Arg1>(f, p, a1);
  }

  template<class T, class Arg1, class Arg2>
  static FunctionCallbackResult<Result>* create(Result (T::*f)(Arg1, Arg2), T* p,
                                             Arg1 a1, Arg2 a2) {
    return new FunctionCallbackResultMember2<Result, T, Arg1, Arg2>(f, p, a1, a2);
  }

  template<class T, class Arg1, class Arg2, class Arg3>
  static FunctionCallbackResult<Result>* create(Result (T::*f)(Arg1, Arg2, Arg3),
                                             T* p, Arg1 a1, Arg2 a2, Arg3 a3) {
    return new FunctionCallbackResultMember3<Result, T, Arg1, Arg2, Arg3>
        (f, p, a1, a2, a3);
  }

  template<class T, class Arg1, class Arg2, class Arg3, class Arg4>
  static FunctionCallbackResult<Result>*
  create(Result (T::*f)(Arg1, Arg2, Arg3, Arg4), T* p,
         Arg1 a1, Arg2 a2, Arg3 a3, Arg4 a4) {
    return new FunctionCallbackResultMember4<Result, T, Arg1, Arg2, Arg3, Arg4>
        (f, p, a1, a2, a3, a4);
  }

  template<class T, class Arg1, class Arg2, class Arg3, class Arg4, class Arg5>
  static FunctionCallbackResult<Result>*
  create(Result (T::*f)(Arg1, Arg2, Arg3, Arg4, Arg5), T* p,
         Arg1 a1, Arg2 a2, Arg3 a3, Arg4 a4, Arg5 a5) {
    return new FunctionCallbackResultMember5<Result, T, Arg1, Arg2, Arg3, Arg4, Arg5>
        (f, p, a1, a2, a3, a4, a5);
  }

  Result result_() {
    return result_;
  }

  void SetResult(Result r) {this->result_ = r;}

 private:

  Result result_;
};

//==============================================================================
//  各コールバックテンプレートの実装
//  各コールバックテンプレートは、原則として全てコピー不可能とする。
//==============================================================================

class FunctionCallbackVoid0 : public FunctionCallbackVoid {
 public:
  typedef void (*CallBack)(void);

  explicit FunctionCallbackVoid0(CallBack c) : function_(c) {}

  virtual void Run() {
    function_();
  }

 private:

  CallBack function_;

};

template<class T>
class FunctionCallbackVoidGlobal0 : public FunctionCallbackVoid {
 public:
  typedef void (T::*CallBack)();

  FunctionCallbackVoidGlobal0(CallBack c) : function_(c) {}
  FunctionCallbackVoidGlobal0(const FunctionCallbackVoidGlobal0& v) : function_(NULL) {
    function_ = v.function_;
  }

  virtual void Run() {
    function_();
  }

 private:

  CallBack function_;

};

template<class Arg1>
class FunctionCallbackVoid1 : public FunctionCallbackVoid {
 public:
  typedef void (*CallBack)(Arg1);

  FunctionCallbackVoid1(CallBack c,Arg1 a1) : function_(c),arg1_(a1) {}
  FunctionCallbackVoid1(const FunctionCallbackVoid1<Arg1>& v) : function_(NULL),arg1_(NULL) {
    function_ = v.function_;
    arg1_ = v.arg1_;
  }

  virtual void Run() {
    function_(arg1_);
  }

 private:

  CallBack function_;
  Arg1 arg1_;

};

template<class Arg1,class Arg2>
class FunctionCallbackVoid2 : public FunctionCallbackVoid {
 public:
  typedef void (*CallBack)(Arg1,Arg2);

  FunctionCallbackVoid2(CallBack c,Arg1 a1,Arg2 a2) : 
      function_(c),arg1_(a1),arg2_(a2) {}
  FunctionCallbackVoid2(const FunctionCallbackVoid2<Arg1,Arg2>& v) :
      function_(NULL),arg1_(NULL),arg2_(NULL) {
    function_ = v.function_;
    arg1_ = v.arg1_;
    arg2_ = v.arg2_;
  }

  virtual void Run() {
    function_(arg1_,arg2_);
  }

 private:

  CallBack function_;
  Arg1 arg1_;
  Arg2 arg2_;

};

template<class Arg1,class Arg2,class Arg3>
class FunctionCallbackVoid3 : public FunctionCallbackVoid {
 public:
  typedef void (*CallBack)(Arg1,Arg2,Arg3);

  FunctionCallbackVoid3(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3) : 
      function_(c),arg1_(a1),arg2_(a2),arg3_(a3) {}
  FunctionCallbackVoid3(const FunctionCallbackVoid3<Arg1,Arg2,Arg3>& v) :
      function_(NULL),arg1_(NULL),arg2_(NULL),arg3_(NULL) {
    function_ = v.function_;
    arg1_ = v.arg1_;
    arg2_ = v.arg2_;
    arg3_ = v.arg3_;
  }

  virtual void Run() {
    function_(arg1_,arg2_,arg3_);
  }

 private:

  CallBack function_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;

};

template<class Arg1,class Arg2,class Arg3,class Arg4>
class FunctionCallbackVoid4 : public FunctionCallbackVoid {
 public:
  typedef void (*CallBack)(Arg1,Arg2,Arg3,Arg4);

  FunctionCallbackVoid4(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) : 
      function_(c),arg1_(a1),arg2_(a2),arg3_(a3),arg4_(a4) {}
  FunctionCallbackVoid4(const FunctionCallbackVoid4
                       <Arg1,Arg2,Arg3,Arg4>& v) :
      function_(NULL),arg1_(NULL),arg2_(NULL),arg3_(NULL),arg4_(NULL) {
    function_ = v.function_;
    arg1_ = v.arg1_;
    arg2_ = v.arg2_;
    arg3_ = v.arg3_;
    arg4_ = v.arg4_;
  }

  virtual void Run() {
    function_(arg1_,arg2_,arg3_,arg4_);
  }

 private:

  CallBack function_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;
  Arg4 arg4_;

};

template<class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
class FunctionCallbackVoid5 : public FunctionCallbackVoid {
 public:
  typedef void (*CallBack)(Arg1,Arg2,Arg3,Arg4,Arg5);

  FunctionCallbackVoid5(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) : 
      function_(c),arg1_(a1),arg2_(a2),arg3_(a3),arg4_(a4),arg5_(a5) {}
  FunctionCallbackVoid5(const FunctionCallbackVoid5<Arg1,Arg2,Arg3,
                       Arg4, Arg5>& v) :
      function_(NULL),arg1_(NULL),arg2_(NULL),arg3_(NULL),
      arg4_(NULL), arg5_(NULL)
  {
    function_ = v.function_;
    arg1_ = v.arg1_;
    arg2_ = v.arg2_;
    arg3_ = v.arg3_;
    arg4_ = v.arg4_;
    arg5_ = v.arg5_;
  }

  virtual void Run() {
    function_(arg1_,arg2_,arg3_,arg4_,arg5_);
  }

 private:

  CallBack function_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;
  Arg4 arg4_;
  Arg5 arg5_;

};

//------------------------------------------------------------------

template<class obj>
class FunctionCallbackVoidMember0 : public FunctionCallbackVoid {
 public:
  typedef void (obj::*CallBack)();

  FunctionCallbackVoidMember0(CallBack c,obj* p) : function_(c),obj_(p) {}
  FunctionCallbackVoidMember0(const FunctionCallbackVoidMember0<obj>& o) : function_(NULL),obj_(NULL)
  {
    function_ = o.function_;
    obj_ = o.obj_;
  }

  virtual void Run() {
    (obj_->*function_)();
  }

 private:

  CallBack function_;
  obj* obj_;

};

template<class obj,class Arg1>
class FunctionCallbackVoidMember1 : public FunctionCallbackVoid {
 public:
  typedef void (obj::*CallBack)(Arg1);

  FunctionCallbackVoidMember1(CallBack c,obj* p,Arg1 a1) : 
      function_(c),obj_(p),arg1_(a1) {}
  FunctionCallbackVoidMember1(const FunctionCallbackVoidMember1<obj,Arg1>& o) :
      function_(NULL),obj_(NULL),arg1_(NULL)
  {
    function_ = o.function_;
    obj_ = o.obj_;
    arg1_ = o.arg1_;
  }

  virtual void Run() {
    (obj_->*function_)(arg1_);
  }

 private:

  CallBack function_;
  obj* obj_;
  Arg1 arg1_;

};

template<class obj,class Arg1,class Arg2>
class FunctionCallbackVoidMember2 : public FunctionCallbackVoid {
 public:
  typedef void (obj::*CallBack)(Arg1,Arg2);

  FunctionCallbackVoidMember2(CallBack c,obj* p,Arg1 a1,Arg2 a2) : 
      function_(c),obj_(p),arg1_(a1),arg2_(a2) {}
  FunctionCallbackVoidMember2(const FunctionCallbackVoidMember2<obj,Arg1,Arg2>& o) :
      function_(NULL),obj_(NULL),arg1_(NULL),arg2_(NULL)
  {
    function_ = o.function_;
    obj_ = o.obj_;
    arg1_ = o.arg1_;
    arg2_ = o.arg2_;
  }

  virtual void Run() {
    (obj_->*function_)(arg1_,arg2_);
  }

 private:

  CallBack function_;
  obj* obj_;
  Arg1 arg1_;
  Arg2 arg2_;

};

template<class obj,class Arg1,class Arg2,class Arg3>
class FunctionCallbackVoidMember3 : public FunctionCallbackVoid {
 public:
  typedef void (obj::*CallBack)(Arg1,Arg2,Arg3);

  FunctionCallbackVoidMember3(CallBack c,obj* p,Arg1 a1,Arg2 a2,Arg3 a3) : 
      function_(c),obj_(p),arg1_(a1),arg2_(a2),arg3_(a3) {}
  FunctionCallbackVoidMember3(const FunctionCallbackVoidMember3<obj,Arg1,Arg2,Arg3>& o) :
      function_(NULL),obj_(NULL),arg1_(NULL),arg2_(NULL),arg3_(NULL)
  {
    function_ = o.function_;
    obj_ = o.obj_;
    arg1_ = o.arg1_;
    arg2_ = o.arg2_;
    arg3_ = o.arg3_;
  }

  virtual void Run() {
    (obj_->*function_)(arg1_,arg2_,arg3_);
  }

 private:

  CallBack function_;
  obj* obj_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;

};

template<class obj,class Arg1,class Arg2,class Arg3,class Arg4>
class FunctionCallbackVoidMember4 : public FunctionCallbackVoid {
 public:
  typedef void (obj::*CallBack)(Arg1,Arg2,Arg3,Arg4);

  FunctionCallbackVoidMember4(CallBack c,obj* p,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) : 
      function_(c),obj_(p),arg1_(a1),arg2_(a2),arg3_(a3),arg4_(a4) {}
  FunctionCallbackVoidMember4(const FunctionCallbackVoidMember4<obj,Arg1,Arg2,Arg3,Arg4>& o) :
      function_(NULL),obj_(NULL),arg1_(NULL),arg2_(NULL),
      arg3_(NULL),arg4_(NULL)
  {
    function_ = o.function_;
    obj_ = o.obj_;
    arg1_ = o.arg1_;
    arg2_ = o.arg2_;
    arg3_ = o.arg3_;
    arg4_ = o.arg4_;
  }

  virtual void Run() {
    (obj_->*function_)(arg1_,arg2_,arg3_,arg4_);
  }

 private:

  CallBack function_;
  obj* obj_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;
  Arg4 arg4_;

};

template<class obj,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
class FunctionCallbackVoidMember5 : public FunctionCallbackVoid {
 public:
  typedef void (obj::*CallBack)(Arg1,Arg2,Arg3,Arg4,Arg5);

  FunctionCallbackVoidMember5(CallBack c,obj* p,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) : 
      function_(c),obj_(p),arg1_(a1),arg2_(a2),arg3_(a3),arg4_(a4),arg5_(a5) {}
  FunctionCallbackVoidMember5(const FunctionCallbackVoidMember5<obj,Arg1,Arg2,Arg3,Arg4,Arg5>& o) :
      function_(NULL),obj_(NULL),arg1_(NULL),arg2_(NULL),
      arg3_(NULL),arg4_(NULL),arg5_(NULL)
  {
    function_ = o.function_;
    obj_ = o.obj_;
    arg1_ = o.arg1_;
    arg2_ = o.arg2_;
    arg3_ = o.arg3_;
    arg4_ = o.arg4_;
    arg5_ = o.arg5_;
  }

  virtual void Run() {
    (obj_->*function_)(arg1_,arg2_,arg3_,arg4_,arg5_);
  }

 private:

  CallBack function_;
  obj* obj_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;
  Arg4 arg4_;
  Arg5 arg5_;

};

//-----------------------------------------------------------------------

template<class Result>
class FunctionCallbackResult0 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (*CallBack)();

  FunctionCallbackResult0(CallBack c) : function_(c) {}
  FunctionCallbackResult0(const FunctionCallbackResult0<Result>& r) :
      function_(NULL)
  {
    function_ = r.function_;
  }

  virtual void Run() {
    Result r = function_();
    SetResult(r);
  }

 private:

  CallBack function_;

};

template<class Result,class T>
class FunctionCallbackResultGlobal0 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (T::*CallBack)();

  FunctionCallbackResultGlobal0(CallBack c) : function_(c) {}
  FunctionCallbackResultGlobal0(const FunctionCallbackResultGlobal0<Result,T>& r) :
      function_(NULL)
  {
    function_ = r.function_;
  }
  FunctionCallbackResult1<Result,T>& operator=(
      const FunctionCallbackResult1<Result, T>& r)
  {
    function_ = r.function_;
  }

  virtual void Run() {
    Result r = function_();
    SetResult(r);
  }

 private:

  CallBack function_;

};

template<class Result,class Arg1>
class FunctionCallbackResult1 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (*CallBack)(Arg1);

  FunctionCallbackResult1(CallBack c,Arg1 a1) : function_(c),arg1_(a1) {}
  FunctionCallbackResult1(const FunctionCallbackResult1<Result,Arg1>& r) :
      function_(NULL), arg1_(NULL)
  {
    function_ = r.function_;
    arg1_ = r.arg1_;
  }
  FunctionCallbackResult1<Result, Arg1>& operator=(
      const FunctionCallbackResult1<Result,Arg1>& r)
  {
    function_ = r.function_;
    arg1_ = r.arg1_;
  }

  virtual void Run() {
    Result r = function_(arg1_);
    SetResult(r);
  }

 private:

  CallBack function_;
  Arg1 arg1_;

};

template<class Result,class Arg1,class Arg2>
class FunctionCallbackResult2 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (*CallBack)(Arg1,Arg2);

  FunctionCallbackResult2(CallBack c,Arg1 a1,Arg2 a2) : function_(c),arg1_(a1),arg2_(a2) {}
  FunctionCallbackResult2(const FunctionCallbackResult2<Result,Arg1,Arg2>& r) :
      function_(NULL), arg1_(NULL), arg2_(NULL)
  {
    function_ = r.function_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
  }
  FunctionCallbackResult2<Result, Arg1, Arg2>& operator=(
      const FunctionCallbackResult2<Result,Arg1,Arg2>& r)
  {
    function_ = r.function_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
  }

  virtual void Run() {
    Result r = function_(arg1_,arg2_);
    SetResult(r);
  }

 private:

  CallBack function_;
  Arg1 arg1_;
  Arg2 arg2_;

};

template<class Result,class Arg1,class Arg2,class Arg3>
class FunctionCallbackResult3 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (*CallBack)(Arg1,Arg2,Arg3);

  FunctionCallbackResult3(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3) : 
      function_(c),arg1_(a1),arg2_(a2),arg3_(a3) {}
  FunctionCallbackResult3(const FunctionCallbackResult3<Result,Arg1,Arg2,Arg3>& r) :
      function_(NULL), arg1_(NULL), arg2_(NULL), arg3_(NULL)
  {
    function_ = r.function_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
    arg3_ = r.arg3_;
  }
  FunctionCallbackResult3<Result, Arg1, Arg2, Arg3>& operator=(
      const FunctionCallbackResult3<Result,Arg1,Arg2, Arg3>& r)
  {
    function_ = r.function_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
    arg3_ = r.arg3_;
  }

  virtual void Run() {
    Result r = function_(arg1_,arg2_,arg3_);
    SetResult(r);
  }

 private:

  CallBack function_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;

};

template<class Result,class Arg1,class Arg2,class Arg3,class Arg4>
class FunctionCallbackResult4 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (*CallBack)(Arg1,Arg2,Arg3,Arg4);

  FunctionCallbackResult4(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) : 
      function_(c),arg1_(a1),arg2_(a2),arg3_(a3),arg4_(a4) {}
  FunctionCallbackResult4(const FunctionCallbackResult4<Result,Arg1,Arg2,Arg3,Arg4>& r) :
      function_(NULL), arg1_(NULL), arg2_(NULL),arg3_(NULL), arg4_(NULL)
  {
    function_ = r.function_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
    arg3_ = r.arg3_;
    arg4_ = r.arg4_;
  }

  virtual void Run() {
    Result r = function_(arg1_,arg2_,arg3_,arg4_);
    SetResult(r);
  }

 private:

  CallBack function_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;
  Arg4 arg4_;

};

template<class Result,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
class FunctionCallbackResult5 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (*CallBack)(Arg1,Arg2,Arg3,Arg4);

  FunctionCallbackResult5(CallBack c,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) : 
      function_(c),arg1_(a1),arg2_(a2),arg3_(a3),arg4_(a4),arg5_(a5) {}
  FunctionCallbackResult5(const FunctionCallbackResult5<Result,Arg1,Arg2,Arg3,Arg4,Arg5>& r) :
      function_(NULL), arg1_(NULL), arg2_(NULL),arg3_(NULL), arg4_(NULL),
      arg5_(NULL)
  {
    function_ = r.function_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
    arg3_ = r.arg3_;
    arg4_ = r.arg4_;
    arg5_ = r.arg5_;
  }


  virtual void Run() {
    Result r = function_(arg1_,arg2_,arg3_,arg4_,arg5_);
    SetResult(r);
  }

 private:

  CallBack function_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;
  Arg4 arg4_;
  Arg5 arg5_;

};

template<class Result,class T>
class FunctionCallbackResultMember0 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (T::*CallBack)();

  FunctionCallbackResultMember0(CallBack c,T* p) : function_(c),obj_(p) {}
  FunctionCallbackResultMember0(const FunctionCallbackResultMember0<Result, T>& r) :
      function_(NULL),obj_(NULL)
  {
    function_ = r.function_;
    obj_ = r.obj_;
  }

  virtual void Run() {
    Result r = (obj_->*function_)();
    SetResult(r);
  }

 private:

  CallBack function_;
  T* obj_;

};

template<class Result,class T,class Arg1>
class FunctionCallbackResultMember1 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (T::*CallBack)(Arg1);

  FunctionCallbackResultMember1(CallBack c,T* p,Arg1 a1) : 
      function_(c),obj_(p),arg1_(a1) {}
  FunctionCallbackResultMember1(const FunctionCallbackResultMember1<Result, T,Arg1>& r) :
      function_(NULL),obj_(NULL), arg1_(NULL)
  {
    function_ = r.function_;
    obj_ = r.obj_;
    arg1_ = r.arg1_;
  }

  virtual void Run() {
    Result r = (obj_->*function_)(arg1_);
    SetResult(r);
  }

 private:

  CallBack function_;
  T* obj_;
  Arg1 arg1_;

};

template<class Result,class T,class Arg1,class Arg2>
class FunctionCallbackResultMember2 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (T::*CallBack)(Arg1,Arg2);

  FunctionCallbackResultMember2(CallBack c,T* p,Arg1 a1,Arg2 a2) : 
      function_(c),obj_(p),arg1_(a1),arg2_(a2) {}
  FunctionCallbackResultMember2(const FunctionCallbackResultMember2<Result,T,Arg1,Arg2>& r) :
      function_(NULL),obj_(NULL), arg1_(NULL), arg2_(NULL)
  {
    function_ = r.function_;
    obj_ = r.obj_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
  }

  virtual void Run() {
    Result r = (obj_->*function_)(arg1_,arg2_);
    SetResult(r);
  }

 private:

  CallBack function_;
  T* obj_;
  Arg1 arg1_;
  Arg2 arg2_;

};

template<class Result,class T,class Arg1,class Arg2,class Arg3>
class FunctionCallbackResultMember3 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (T::*CallBack)(Arg1,Arg2,Arg3);

  FunctionCallbackResultMember3(CallBack c,T* p,Arg1 a1,Arg2 a2,Arg3 a3) : 
      function_(c),obj_(p),arg1_(a1),arg2_(a2),arg3_(a3) {}
  FunctionCallbackResultMember3(const FunctionCallbackResultMember3<Result, T,
                        Arg1,Arg2,Arg3>& r) :
      function_(NULL),obj_(NULL), arg1_(NULL), arg2_(NULL),
      arg3_(NULL)
  {
    function_ = r.function_;
    obj_ = r.obj_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
    arg3_ = r.arg3_;
  }

  virtual void Run() {
    Result r = (obj_->*function_)(arg1_,arg2_,arg3_);
    SetResult(r);
  }

 private:

  CallBack function_;
  T* obj_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;

};

template<class Result,class T,class Arg1,class Arg2,class Arg3,class Arg4>
class FunctionCallbackResultMember4 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (T::*CallBack)(Arg1,Arg2,Arg3,Arg4);

  FunctionCallbackResultMember4(CallBack c,T* p,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4) : 
      function_(c),obj_(p),arg1_(a1),arg2_(a2),arg3_(a3),arg4_(a4) {}
  FunctionCallbackResultMember4(const FunctionCallbackResultMember4<Result, T,
                        Arg1,Arg2,Arg3,Arg4>& r) :
      function_(NULL),obj_(NULL), arg1_(NULL), arg2_(NULL),
      arg3_(NULL), arg4_(NULL)
  {
    function_ = r.function_;
    obj_ = r.obj_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
    arg3_ = r.arg3_;
    arg4_ = r.arg4_;
  }

  virtual void Run() {
    Result r = (obj_->*function_)(arg1_,arg2_,arg3_,arg4_);
    SetResult(r);
  }

 private:

  CallBack function_;
  T* obj_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;
  Arg4 arg4_;

};

template<class Result,class T,class Arg1,class Arg2,class Arg3,class Arg4,class Arg5>
class FunctionCallbackResultMember5 : public FunctionCallbackResult<Result> {
 public:
  typedef Result (T::*CallBack)(Arg1,Arg2,Arg3,Arg4,Arg5);

  FunctionCallbackResultMember5(CallBack c,T* p,Arg1 a1,Arg2 a2,Arg3 a3,Arg4 a4,Arg5 a5) : 
      function_(c),obj_(p),arg1_(a1),arg2_(a2),arg3_(a3),arg4_(a4),arg5_(a5) {}
  FunctionCallbackResultMember5(const FunctionCallbackResultMember5<Result, T,
                        Arg1,Arg2,Arg3,Arg4, Arg5>& r) :
      function_(NULL),obj_(NULL), arg1_(NULL), arg2_(NULL),
      arg3_(NULL), arg4_(NULL), arg5_(NULL)
  {
    function_ = r.function_;
    obj_ = r.obj_;
    arg1_ = r.arg1_;
    arg2_ = r.arg2_;
    arg3_ = r.arg3_;
    arg4_ = r.arg4_;
    arg5_ = r.arg5_;
  }

  virtual void Run() {
    Result r = (obj_->*function_)(arg1_,arg2_,arg3_,arg4_,arg5_);
    SetResult(r);
  }

 private:

  CallBack function_;
  T* obj_;
  Arg1 arg1_;
  Arg2 arg2_;
  Arg3 arg3_;
  Arg4 arg4_;
  Arg5 arg5_;

};


// この関数のみ、実体が必要であるため、ここで定義する。
inline FunctionCallbackVoid* create(void (*f)(void))
{
  return new FunctionCallbackVoid0(f);
}


};

#endif
