#ifndef _DATA_STRUCTURE_H_
#define _DATA_STRUCTURE_H_

#include <algorithm>
#include "src/common/smart_ptr.h"

namespace utakata {

namespace type {
class TypeDescripter;
};

namespace data {

class DataSpace {
  // データ領域を表すだけのクラス。
  // データ領域の入替などを、このクラス自体への代入で済ますために、
  // オペレータオーバロードが実装されている。
 public:
  explicit DataSpace(const smart_ptr<type::TypeDescripter>& data);
  DataSpace(const DataSpace& space);
  virtual ~DataSpace() {}

  DataSpace& operator=(const DataSpace& space);
  bool operator==(const DataSpace& space);
  bool operator!=(const DataSpace& space);

  type::TypeDescripter& getTypeDescripter();

 private:

  // データ領域を交換する。
  void swap(DataSpace& space);

  // データの実体を保持するTypeDescripterのsmart_ptr
  smart_ptr<type::TypeDescripter> space_;
};
};
};

#endif /* _DATA_STRUCTURE_H_ */
