#ifndef _DATA_STRUCTURE_INTERFACE_H_
#define _DATA_STRUCTURE_INTERFACE_H_

#include <string>
#include "src/common/smart_ptr.h"

namespace utakata {

namespace unicode {
class UniString;
};

namespace interpreter {
class IObject;
};

namespace data {

class DataEntity {
  // 各型データの実体アドレスを、void*型に変換して返す。
  // 単純な機能ではあるが、実体に対する最後の抽象インターフェースである
 public:
  DataEntity() {}
  virtual ~DataEntity() {}

  virtual void* getAddress() = 0;
  virtual const void* getAddress() const = 0;
};

class DataEntityImpl : public DataEntity {
  // DataEntityを定義した実体。この実体は、IObjectをsmart_ptrとして
  // 保持し、解体責任を持つ。
 public:

  explicit DataEntityImpl(interpreter::IObject* obj) : obj_(obj, true) {}
  virtual ~DataEntityImpl() {}

  virtual void* getAddress() {return static_cast<void*>(&(*obj_));}
  virtual const void* getAddress() const {
    return static_cast<const void*>(&(*obj_));
  }

 private:

  smart_ptr<interpreter::IObject> obj_;
};
};
};

#endif /* _DATA_STRUCTURE_INTERFACE_H_ */
