#ifndef _DATUM_ID_H_
#define _DATUM_ID_H_

// datumで示される構文データの各ID

#include <string>

namespace utakata {

namespace syntax {

class DatumID {
 public:
  enum private_DATUMID_ {
    STRING        = 0,                  // <string>
    ABBREVIATIONS = 1,                  // 各abbrev prefix
    SYMBOL        = 2,                  // <symbol>
    NUMBER        = 3,                  // <number>
    CHARACTOR     = 4,                  // <charactor>
    BOOLEAN       = 5,                  // <boolean>
    BYTEVECTOR    = 6,                  // <bytevector>
    VECTOR        = 7,                  // <vector>
    LIST          = 8,                  // <list>
    LITERAL       = 9,                  // <lexeme datum>
    NIL           = 10,
  };

 private:

  typedef private_DATUMID_ DATUMID_;
  DATUMID_ S_;

  template<DATUMID_> class Literal;

 public:

  template<DATUMID_ S>
  explicit DatumID(const Literal<S>&) : S_(S) {}

  DATUMID_ toEnum() const {return S_;}

  friend bool operator==(DatumID s, DatumID t) {return s.S_ == t.S_;}
  friend bool operator!=(DatumID s, DatumID t) {return !(s == t);}

  static const Literal<STRING>           string;
  static const Literal<ABBREVIATIONS>    abbreviations;
  static const Literal<SYMBOL>           symbol;
  static const Literal<NUMBER>           number;
  static const Literal<CHARACTOR>        charactor;
  static const Literal<BOOLEAN>          boolean;
  static const Literal<BYTEVECTOR>       byteVector;
  static const Literal<VECTOR>           vector;
  static const Literal<LIST>             list;
  static const Literal<LITERAL>       literal;
  static const Literal<NIL>              nil;
};

// 内部クラスかつ、単に該当するEnumを返すだけのクラステンプレート
template<DatumID::private_DATUMID_ S>
class DatumID::Literal : private DatumID {
 public:

  DatumID::private_DATUMID_ toEnum() const {return S;}

 private:

  friend class DatumID;
  // 暗黙的なコピーコンストラクタを利用する。
  Literal() : DatumID(*this) {}
  ~Literal() {}

  void* operator new(size_t size);
  void operator delete(void* pointer);
  void operator=(const Literal& literal);
  void* operator&() const;
};
};
};

#endif /* _DATUM_ID_H_ */
