#ifndef _DELIMITER_H_
#define _DELIMITER_H_

#include "src/unicode.h"
#include "src/common/smart_ptr.h"

namespace utakata {

namespace unicode {
class UniChar;
};

namespace reader {
class StreamReader;
};

namespace lexer_delimiter {

//==============================================================================
// 各デリミタを判別するための関数オブジェクト。
//==============================================================================

class Normal {
  // 通常のデリミタとして処理するべきデリミタ一覧を返す。
 public:
  bool operator()(const utakata::unicode::UniChar& ch);
};

class Whitespace {
  // 空白として認識される文字であるかどうかを返す。
 public:
  bool operator()(const utakata::unicode::UniChar& ch);
};

class LineEnding {
  // 改行と認識される文字であるかどうかを返す。
  // 但し、次の文字も含め改行である場合があるため、
  // まとめて調べるため、streamを必要とする。
 public:
  bool operator()(const utakata::unicode::UniChar& ch,
                  smart_ptr<reader::StreamReader>& strm);
};

class String {
  // 文字列を分割するためのデリミタを定義する。
 public:
  bool operator()(const utakata::unicode::UniChar& ch);
};

class Prefix {
  // <number>のプレフィックスを調査する。
 public:
  bool operator()(const utakata::unicode::UniString& str);
};

class Exactness {
  // <number>の正確性を調査する。
 public:
  bool operator()(const utakata::unicode::UniString& str);
};

class HexValue {
  // 16進数で利用可能な文字の範囲であるかどうかを返す。
 public:
  bool operator()(const utakata::unicode::UniChar& ch);
};
};
};
#endif /* _DELIMITER_H_ */
