#include <utility>
#include "environment.h"
#include "object.h"

using namespace utakata::data;
using namespace utakata::interpreter;

Environment::Environment() : bind_(), parent_()
{
    // このコンストラクタは、事実上ルート環境でしか利用されることはない。
}

Environment::Environment(const smart_ptr<Environment>& parent) :
    bind_(),
    parent_(parent)
{
}

void Environment::append(const std::string& key, smart_ptr<Object> o)
{
    // 指定したkeyに、Objectを束縛する。
    // すでに該当する値が現環境に存在する場合には、上書きされる。
    if (bind_.find(key) == bind_.end())
    {
        bind_.insert(std::pair<std::string, smart_ptr<Object> >(
                         key, o));
    }
    else
    {
        // 上書きは単純なアクセスで行える。
        bind_[key] = o;
    }
}

smart_ptr<Object> Environment::operator[](const std::string& str)
{
    // キー値に束縛されている値を返す。
    // 束縛されている値が存在しない場合は、親に遡って検索を行う。

    if (bind_.find(str) == bind_.end() && !parent_.isNull())
    {
        // 親に遡って検索を行う。
        smart_ptr<Environment> p = parent_;
        smart_ptr<Object> o;
        while (!p.isNull() && o.isNull())
        {
            o = p->find(str);
            p = p->parent_;
        }
        // 親がそれ以上辿れなかったり、発見された場合には辿るのを止める。
        return o;
    }
    else
    {
        return bind_.find(str)->second;
    }
}

smart_ptr<Object> Environment::find(const std::string& str)
{
    // 自身のみの検索を行う。この関数はEnvironmentのみで実行することができる。
    if (bind_.find(str) == bind_.end())
    {
        return smart_ptr<Object>();
    }
    else
    {
        return bind_.find(str)->second;
    }
}
