#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_

#include <map>
#include <string>
#include "src/common/smart_ptr.h"

namespace utakata {

namespace data {
class Object;
};

namespace interpreter {

class Environment {
  // Scheme中の「環境」とされる束縛の集合を表す。
  // 親となる環境へのポインタを保持し、順に束縛を検索していく。
  // 束縛は文字列をキーとしたObjectのmapで表現され、
  // 実際の利用に際しては、Literalのsymbolから取得できる文字列を
  // 元にしてアクセスを行う。
 public:

  Environment();
  explicit Environment(const smart_ptr<Environment>& parent);
  virtual ~Environment() {}

  // 指定した名前にObjectを束縛する。
  void append(const std::string& key, smart_ptr<data::Object> obj);

  // 基本的にはこれのみでアクセスできるようにする。
  // 存在しない束縛にアクセスした場合には、isNullがNULLとなる
  // smart_ptrを返す(つまりはNULLオブジェクト)
  smart_ptr<data::Object> operator[](const std::string& str);

 private:

  smart_ptr<data::Object> find(const std::string& str);

  std::map<std::string, smart_ptr<data::Object> > bind_;

  // 現在の環境の親となる環境。
  smart_ptr<Environment> parent_;
};
};
};
#endif /* _ENVIRONMENT_H_ */
