#include "gc.h"
#include "type.h"
#include "base_object.h"
#include "data_structure.h"
#include "data_structure_interface.h"
#include "class.h"

using namespace utakata;
using namespace utakata::gc;

smart_ptr<data::DataSpace> gc::GarbageCollector::add(interpreter::IObject* o)
{
    // 引数として渡されたオブジェクトを、gcが唯一の管理元として管理する。
    // この管理の際、オブジェクトのtypeも一度だけ登録される。

    if (!isRegistered(o))
    {
        registerType(o);
    }

    // まずはDataEntityを作成する。
    // DataEntityは統一されたインターフェースで、他のデータを利用できるようにして、
    // 後は管理しやすくする。
    smart_ptr<data::DataEntity> ent(new data::DataEntityImpl(o));

    // 領域に追加する。
    spaces_.push_back(ent);

    // Typeのmapから、該当するtype::Typeを取得して、TypeDescripterと
    // Dataspaceを生成する。
    smart_ptr<type::TypeDescripter> td(new type::TypeDescripter(
                                           *(types_[o->getClass()->class_name()]),
                                           ent));
    smart_ptr<data::DataSpace> space(new data::DataSpace(td));

    return space;
}

bool gc::GarbageCollector::isRegistered(interpreter::IObject* o)
{
    // すでに登録されているかどうかを返す。
    if (types_.find(o->getClass()->class_name()) == types_.end())
    {
        return false;
    }
    else
    {
        return true;
    }
}

void gc::GarbageCollector::registerType(interpreter::IObject* o)
{
    // type::Typeを登録する。

    std::pair<std::string, smart_ptr<type::Type> > p;
    p.first = o->getClass()->class_name();
    p.second = smart_ptr<type::Type>(new type::Type(o->getClass()->class_name()));

    types_.insert(p);
}

void gc::GarbageCollector::collect()
{
    // 現在保持しているspacesの中で、spaces_からのみ参照されているデータの
    // 削除を行う。

    std::vector<smart_ptr<data::DataEntity> >::iterator it = spaces_.begin(),
        end = spaces_.end();

    for (; it != end;)
    {
        if (it->isOnlyRef())
        {
            // カウントが1の場合、削除する。
            it = spaces_.erase(it);
        }
        else
        {
            ++it;
        }
    }

}
