#ifndef _GC_H_
#define _GC_H_

#include <string>
#include <vector>
#include <map>
#include "src/common/smart_ptr.h"

namespace utakata {

namespace data {
class DataSpace;
class DataEntity;
};

namespace type {
class Type;
};

namespace interpreter {
class IObject;
};

namespace gc {

class GarbageCollector {
  // シンプルなガベージコレクター。scheme中の全てのオブジェクトは、
  // このGarbageCollectorが保持し、ここから取得することになる。
 public:

  GarbageCollector() {}
  virtual ~GarbageCollector() {}

  // 渡されたオブジェクトを管理下に置く。
  // この時渡されるオブジェクトは、基本的にClassオブジェクトから
  // 生成されるオブジェクトになる。
  smart_ptr<data::DataSpace> add(interpreter::IObject* o);

  // 現在無効であるオブジェクトを削除する。
  void collect();

 private:

  bool isRegistered(interpreter::IObject* o);
  void registerType(interpreter::IObject* o);

  // 実際のデータを格納するDataEntity
  std::vector<smart_ptr<data::DataEntity> > spaces_;

  // TypeDescripterを内部で生成する際に必要な、各型情報を
  // 一意で保存するためのType。
  // 実際には、ここから一意な型情報の参照が取られる。
  std::map<std::string, smart_ptr<type::Type> > types_;
};
};
};

#endif /* _GC_H_ */
