#ifndef _LEXEME_H_
#define _LEXEME_H_

#include "src/common/disable_copy.h"
#include "src/common/smart_ptr.h"

namespace utakata {

namespace unicode {
class UniString;
};

namespace lexeme {

class LexemeID;
struct LexemeData;
class ILexeme : private utility::DisableCopyInterface {
  // 非終端記号、及び終端記号を表すクラス。
  // それぞれを取得するためのインターフェースはこれから派生して作成される。
 public:
  virtual ~ILexeme() {}

  // 終端記号、非終端記号のIDを取得する。
  virtual const LexemeID getID() const = 0;

  // stringのデータ型において、文字列を取得する。
  virtual smart_ptr<utakata::unicode::UniString> toString() const = 0;

  // リテラルとして扱うためのデータを取得する。各リテラルのうち、
  // lexeme Datumとして扱うことができるデータについてはこれらとなる。
  virtual smart_ptr<LexemeData> getData() const = 0;
};
};
};

#endif /* _LEXEME_H_ */
