#include "lexeme_data.h"
#include "unicode.h"

using namespace utakata::lexeme;
using namespace utakata::unicode;

smart_ptr<LexemeData> utakata::lexeme::makeStringData(const unicode::UniString& str)
{
    // stringを内部に生成して返すのみ。
    smart_ptr<LexemeData> ret(new LexemeData);
    ret->string.add(new StringData);
    ret->string->str.add(new UniString(str));
    return ret;
}

smart_ptr<LexemeData> utakata::lexeme::makeNumberData(const unicode::UniString& real,
                                                  const unicode::UniString& imagin,
                                                  bool exact, bool radix)
{
    // numberを内部に生成して返すのみ。
    smart_ptr<LexemeData> ret(new LexemeData);
    ret->number.add(new NumberData);
    ret->number->real.add(new UniString(real));
    ret->number->imagin.add(new UniString(imagin));
    ret->number->exact = exact;
    ret->number->radix = radix;

    return ret;
}

smart_ptr<LexemeData> utakata::lexeme::makeBooleanData(const unicode::UniString& str,
                                                   bool boolean)
{
    // booleanを内部に生成して返すのみ。
    smart_ptr<LexemeData> ret(new LexemeData);
    ret->boolean.add(new BooleanData);
    ret->boolean->str.add(new UniString(str));
    ret->boolean->boolean = boolean;
    return ret;
}

smart_ptr<LexemeData> utakata::lexeme::makeCharactorData(const unicode::UniChar& ch)
{
    // charactorを内部に生成する。charactorは、元となる文字列から、
    // 対応するUTF-8の文字コードを生成する。
    smart_ptr<LexemeData> ret(new LexemeData);
    ret->charactor.add(new CharactorData);
    ret->charactor->spec.add(new UniChar(ch));
    return ret;
}

smart_ptr<LexemeData> utakata::lexeme::makeSymbolData(const unicode::UniString& str)
{
    // symnbolを内部に生成して返すのみ。
    smart_ptr<LexemeData> ret(new LexemeData);
    ret->symbol.add(new SymbolData);
    ret->symbol->id.add(new UniString(str));
    return ret;
}
