#include <iostream>
#include <sstream>

#include <assert.h>

#include "lexer.h"
#include "sublexer_impl.h"
#include "lexeme.h"
#include "reader.h"


using namespace utakata;

smart_ptr<lexeme::ILexeme> lexer::Lexer::lex(smart_ptr<reader::StreamReader>& stream)
{
    // 渡されたCUTF8InputStreamから、1文字ずつ読んでいき、各構文を解釈
    // する。

    // 何か一つの非終端記号、終端記号を読みだすたびにそれを返す。

    // まずは何はなくとも1文字読みだす。

    // EOF以外の値の場合には、通常の字句解析を行っていく。
    smart_ptr<sublexer::ISubLexer> sublex(new sublexer::FirstLexer());

    try {
    while (!stream->isEOF() && !sublex.isNull()) {

        // 基本的に、sublexerに全処理を委譲する形になる。
        // sublexerはそれぞれ別々のデリミタfunctorを持っており
        // それぞれのデリミタfunctorで、各デリミタを判別する。
        // 最初を含む構文チェックはFirstLexerが行ない、それ以外なら
        // そのまま行なう。
        
        smart_ptr<sublexer::ISubLexer> next;
        smart_ptr<lexeme::ILexeme> l = sublex->lex(stream, next);
        if (next.isNull())
        {
            return l;
        }
        // 次に移れるようにしておく。
        sublex = next;
    }
    } catch (std::exception& e)
    {
        std::cout << "exception position : " << stream->pos() << std::endl;
    }

    // 何もなく終了してしまった場合、これはエラーとなる。
    return smart_ptr<lexeme::ILexeme>();
}

