#ifndef _LEXER_H_
#define _LEXER_H_

#include <exception>
#include <string>
#include "src/common/smart_ptr.h"

namespace utakata {

namespace lexeme {
class ILexeme;
};

namespace reader {
class StreamReader;
};

namespace lexer {

class Lexer {
  /**
     scheme構文の字句解析器。
     単純な字句解析器として働く。ストリームから文字列を読みだし、
     BNF構文に基づいたそれぞれのオブジェクトに変換される。
     Schemeの構文解析処理は、どちらかというとこの字句解析が大半を占める。
     それぞれの字句解析は、該当するオブジェクトに変換される。
     処理としてはそれほど難しくはない、と思いたい。

     基本方針としては、シンプル->リファクタリングを繰り返すことを基本と
     する。なので、最初はlex関数一つしか無いが、状況に応じて随時メンバ
     関数を増やしたりfunctorとして分離させたりする。でも基本的にこれひ
     とつで済むようにした方が楽かもしんない。
  */
 public:
  Lexer() {}
  virtual ~Lexer() {}

  /**
     渡されたUTF8を解釈するstreamから、データを解釈して、結果を返す。
     結果は、smart_ptr<CLexeme>で返される。
  */
  smart_ptr<utakata::lexeme::ILexeme>
  lex(smart_ptr<utakata::reader::StreamReader> stream);
};
};                                      // end of namespace `lexer`
};                                      // end of namespace `lexer`

#endif /* _LEXER_H_ */
