#include "object.h"
#include "type.h"

using namespace utakata;

data::Object::Object(bool changeable, const smart_ptr<DataSpace>& space) :
    space_(space), changeable_(changeable)
{
}

data::Object::Object(const data::Object& d)
{
    swap(d);
}

data::Object& data::Object::operator=(const data::Object& o)
{
    data::Object tmp(o);
    swap(tmp);
    return *this;
}

bool data::Object::enableChange() const
{
    return changeable_;
}

const smart_ptr<data::DataSpace> data::Object::getSpace() const
{
    return space_;
}

void data::Object::setSpace(const smart_ptr<data::DataSpace>& space)
{
    // 現在保持しているspaceを入れ替える。
    // ここで入れ替えるのは、束縛する値を変更している(define)を指す。
    space_ = space;
}

void data::Object::swap(const data::Object& o)
{
    space_ = o.space_;
    changeable_ = o.changeable_;
}

bool data::instanceof(const data::Object& o, const type::Type& type)
{
    // Objectが保有するTypeと渡されたtypeが等しいかどうかを検査する。
    return o.getSpace()->getTypeDescripter().getType() == type;
}
