#ifndef _PARSER_HANDLER_H_
#define _PARSER_HANDLER_H_

#include "smart_ptr.h"
#include <string>
#include <stack>
#include <vector>
#include "parser_status.h"

namespace utakata {

    namespace lexeme {
        class ILexeme;
    };

    namespace data {
        class Object;
    };

    namespace gc {
        class GarbageCollector;
    };

    namespace parser {

        struct HandlerData
        {
            HandlerData(gc::GarbageCollector& g) : gc(g), chain(false) {}

            smart_ptr<lexeme::ILexeme> lexm;
            std::stack<data::Object> stack;
            std::stack<std::vector<data::Object> > compounds;
            gc::GarbageCollector& gc;
            std::stack<PARSERSTATUS> status;
            std::string error;   // syntax errorなどを通知する。
            bool chain;          // 現在のlexemeを維持したまま次のstatusに基づいた処理を行う。
        };
        
        class IParserHandler
        {
        public:
            
            IParserHandler() {}
            virtual ~IParserHandler() {}

            // 構文木のスタックとstatusのスタックを元に、Treeを
            // 構築する。
            bool exec(HandlerData& data);

        private:

            // 実際に起動される関数。
            // 処理が行われた場合にはtrue、行われなかった場合にはfalseを
            // 返すこと。
            virtual bool exec_(HandlerData& data) = 0;

        };

    };

};

#endif /* _PARSER_HANDLER_H_ */
