#include "object.h"
#include "primitive.h"
#include "unicode.h"
#include "primitive_class.h"
#include "number.h"

using namespace utakata;
using namespace utakata::interpreter;
using namespace utakata::unicode;

interpreter::primitive::String::String(const smart_ptr<UniString>& data) : data_(data)
{
}

UniString& interpreter::primitive::String::getData()
{
    return *data_;
}

smart_ptr<interpreter::IClass> interpreter::primitive::String::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<String>());
}

interpreter::primitive::Charactor::Charactor(const smart_ptr<UniString>& data) : data_()
{
    // UniStringを解析して、文字コードに変換する。
    // ここで渡されるdataは、すでに先頭の#\が取り除かれている状態となっている。
    // 実際にはここで解釈するのではなく、
}

UniChar& interpreter::primitive::Charactor::getData()
{
    return *data_;
}

smart_ptr<interpreter::IClass> interpreter::primitive::Charactor::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Charactor>());
}


interpreter::primitive::Number::Number(const smart_ptr<unicode::UniString>& real,
                                       const smart_ptr<unicode::UniString>& imagin,
                                       bool exact, int radix) :
    num_ent_()
{
    // 渡されたreal、imagin、exact、radixによってそれぞれ処理が微妙に異なる。
    // radixなどは、基数変換を行う際に必要となる。
    // 詳細はnumber.hを参照。
}

smart_ptr<interpreter::IClass> interpreter::primitive::Number::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Number>());
}

interpreter::primitive::Boolean::Boolean(bool b) : boolean_(b)
{
}

bool interpreter::primitive::Boolean::getData() const
{
    return boolean_;
}

smart_ptr<interpreter::IClass> interpreter::primitive::Boolean::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Boolean>());
}


interpreter::primitive::Symbol::Symbol(const smart_ptr<unicode::UniString>& sym) :
    symbol_(sym)
{
}

unicode::UniString& interpreter::primitive::Symbol::getData()
{
    return *symbol_;
}

smart_ptr<interpreter::IClass> interpreter::primitive::Symbol::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Symbol>());
}

interpreter::primitive::Cons::Cons(const data::Object& car,
                        const data::Object& cdr) :
    car_(new data::Object(car)), cdr_(new data::Object(cdr))
{
}

data::Object& interpreter::primitive::Cons::car()
{
    return *car_;
}

data::Object& interpreter::primitive::Cons::cdr()
{
    return *cdr_;
}

void interpreter::primitive::Cons::setCar(const data::Object& obj)
{
    // デフォルトでNullObjectが代入されているため、そのまま
    // 設定することが可能になっている。
    *car_ = obj;
}

void interpreter::primitive::Cons::setCdr(const data::Object& obj)
{
    // なんらかの値が設定されている場合には破棄する。
    *cdr_ = obj;
}

smart_ptr<interpreter::IClass> interpreter::primitive::Cons::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Cons>());
}

interpreter::primitive::ByteVector::ByteVector(const std::vector<data::Object>& vec) :
    vector_(vec)
{
}

std::vector<data::Object>& interpreter::primitive::ByteVector::getData()
{
    return vector_;
}

smart_ptr<interpreter::IClass> interpreter::primitive::ByteVector::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<ByteVector>());
}


interpreter::primitive::Vector::Vector(const std::vector<data::Object>& vec) :
    vector_(vec)
{
}

std::vector<data::Object>& interpreter::primitive::Vector::getData()
{
    return vector_;
}

smart_ptr<interpreter::IClass> interpreter::primitive::Vector::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Vector>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::Nil::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Nil>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::Quote::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Quote>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::Quasiquote::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Quasiquote>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::Unquote::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Unquote>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::UnquoteSplicing::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<UnquoteSplicing>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::Syntax::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Syntax>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::Quasisyntax::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Quasisyntax>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::Unsyntax::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<Unsyntax>());
}

smart_ptr<interpreter::IClass> interpreter::primitive::UnsyntaxSplicing::getClass()
{
    using namespace utakata::interpreter::primitive;
    return smart_ptr<interpreter::IClass>(new PrimitiveClass<UnsyntaxSplicing>());
}
