#ifndef _PRIMITIVE_CLASS_H_
#define _PRIMITIVE_CLASS_H_

#include "class.h"
#include "primitive.h"
#include "primitive_traits.h"

// primitiveのための各クラスを作成する。

namespace utakata {
    namespace interpreter {
        namespace primitive {

            template<class T>
            class PrimitiveClass : public Class<T>
            {
            public:
                std::string class_name() const {
                    return primitive_traits<T>::name();
                }
            };

            template<>
            struct primitive_traits<String>
            {
                static std::string name() {return "string";}
            };

            template<>
            struct primitive_traits<Charactor>
            {
                static std::string name() {return "charactor";}
            };

            template<>
            struct primitive_traits<Number>
            {
                static std::string name() {return "number";}
            };

            template<>
            struct primitive_traits<Boolean>
            {
                static std::string name() {return "boolean";}
            };

            template<>
            struct primitive_traits<Cons>
            {
                static std::string name() {return "cons";}
            };

            template<>
            struct primitive_traits<ByteVector>
            {
                static std::string name() {return "bytevector";}
            };

            template<>
            struct primitive_traits<Symbol>
            {
                static std::string name() {return "symbol";}
            };

            template<>
            struct primitive_traits<Vector>
            {
                static std::string name() {return "vector";}
            };

            template<>
            struct primitive_traits<Nil>
            {
                static std::string name() {return "nil";}
            };

            template<>
            struct primitive_traits<Quote>
            {
                static std::string name() {return "quote";}
            };


            template<>
            struct primitive_traits<Quasiquote>
            {
                static std::string name() {return "quasiquote";}
            };

            template<>
            struct primitive_traits<Unquote>
            {
                static std::string name() {return "unquote";}
            };

            template<>
            struct primitive_traits<UnquoteSplicing>
            {
                static std::string name() {return "unquote-splicing";}
            };

            template<>
            struct primitive_traits<Syntax>
            {
                static std::string name() {return "syntax";}
            };

            template<>
            struct primitive_traits<Quasisyntax>
            {
                static std::string name() {return "quasisyntax";}
            };

            template<>
            struct primitive_traits<Unsyntax>
            {
                static std::string name() {return "unsyntax";}
            };

            template<>
            struct primitive_traits<UnsyntaxSplicing>
            {
                static std::string name() {return "unsytnax-splicing";}
            };

            // define each primtive class typedef's
            typedef PrimitiveClass<String> StringClass;
            typedef PrimitiveClass<Number> NumberClass;
            typedef PrimitiveClass<Charactor> CharactorClass;
            typedef PrimitiveClass<Symbol> SymbolClass;
            typedef PrimitiveClass<Boolean> BooleanClass;
            typedef PrimitiveClass<Cons> ConsClass;
            typedef PrimitiveClass<ByteVector> ByteVectorClass;
            typedef PrimitiveClass<Vector> VectorClass;
            typedef PrimitiveClass<Nil> NilClass;
            typedef PrimitiveClass<Quote> QuoteClass;
            typedef PrimitiveClass<Unquote> UnqucoteClass;
            typedef PrimitiveClass<Quasiquote> QuasiquoteClass;
            typedef PrimitiveClass<UnquoteSplicing> UnquoteSplicingClass;
            typedef PrimitiveClass<Syntax> SyntaxClass;
            typedef PrimitiveClass<Unsyntax> UnsyntaxClass;
            typedef PrimitiveClass<Quasisyntax> QuasisyntaxClass;
            typedef PrimitiveClass<UnsyntaxSplicing> UnsyntaxSplicingClass;

        };  /* end of namespace `primitive` */
    }; // end of namespace `interpreter`
}; // end of namespace `utakata`

#endif /* _PRIMITIVE_CLASS_H_ */
