#ifndef _PRIMITIVE_UTIL_H_
#define _PRIMITIVE_UTIL_H_

#include <vector>
#include "primitive.h"

namespace utakata {

    namespace data {
        class Object;
    };

    namespace unicode {
        class UniString;
        class UniChar;
    };

    namespace interpreter {
        class IObject;
        namespace util {
            ///////////////////////////////////////////////
            // implementation of create primitive object //
            ///////////////////////////////////////////////

            // 各プリミティブオブジェクトを生成するための各関数。
            interpreter::IObject* makeCons(const data::Object& car,
                                           const data::Object& cdr);
            interpreter::IObject* makeNumber(const smart_ptr<unicode::UniString>& real,
                                             const smart_ptr<unicode::UniString>& imagin,
                                             bool exact, int radix);
            interpreter::IObject* makeNil();
            interpreter::IObject* makeString(const smart_ptr<unicode::UniString>& data);
            interpreter::IObject* makeCharactor(const smart_ptr<unicode::UniChar>& data);
            interpreter::IObject* makeBoolean(bool b);
            interpreter::IObject* makeSymbol(const smart_ptr<unicode::UniString>& data);
            interpreter::IObject* makeByteVector(const std::vector<data::Object>& vec);
            interpreter::IObject* makeVector(const std::vector<data::Object>& vec);

            // 各種abbreviationを作成する。
            interpreter::IObject* makeQuote();
            interpreter::IObject* makeBackQuote();
            interpreter::IObject* makeQuote();
            interpreter::IObject* makeUnquote();
            interpreter::IObject* makeUnquoteSplicing();
            interpreter::IObject* makeSyntax();
            interpreter::IObject* makeQuasisyntax();
            interpreter::IObject* makeUnsyntax();
            interpreter::IObject* makeUnsyntaxSplicing();

            template<class T>
            struct Abbrev
            {
                static interpreter::IObject* make() {
                    return new T;
                }
            };

            // 各プリミティブオブジェクト専用のユーティリティ関数

            // 渡されたconsのcdrに連なるオブジェクトの内、末尾であるオブジェクト
            // を返す。
            data::Object& getLastCons(data::Object& c);

            // cの末尾に、渡されたdatを追加する。
            // cがconsでは無い場合、例外を発生させる。
            void concat(data::Object& c, const data::Object& dat);

        }; // end of namespace util
    };
};

#endif /* _PRIMITIVE_UTIL_H_ */
