#include <iostream>
#include <sstream>
#include <string>
#include <functional>

#include "../simpletest.h"

#include "../bignum.h"

using namespace utakata;

bool bignum_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    // メモリの許す限り桁数を維持する整数オブジェクトのテストを行う。
    numeric::Bignum a("1000");
    numeric::Bignum b("2000");

    numeric::Bignum c;

    asserter->check(a < b, true , "failure less than");
    asserter->check(a > b, false, "failure greater than");
    asserter->check(a <= b, true, "failure less equal");
    asserter->check(a >= b, false, "failure greater equal");
    asserter->check(a == b, false, "failure equal");
    asserter->check(a != b, true, "failure not equal");
    c = a + b;
    asserter->check(a.to_str(), "1000", "error to_str 1000");
    asserter->check(b.to_str(), "2000", "error to_str 2000");
    asserter->check(c.to_str(), "3000", "error to_str 3000");

    c = a - b;
    asserter->check(c.to_str(), "-1000");
    c = c + a;
    asserter->check(c.to_str(), "0");

    c = a * b;
    asserter->check(c.to_str(), "2000000");

    c = a / b;
    asserter->check(c.to_str(), "0");
    c = b / a;
    asserter->check(c.to_str(), "2");

    // 巨大数の取り扱い。
    // 1000億
    a = numeric::Bignum("100000000000");
    b = numeric::Bignum("200100200032");

    c = a + b;
    asserter->check(c.to_str(), "300100200032", "failure plus result not equal 300100200032");

    c = a - b;
    asserter->check(c.to_str(), "-100100200032", "failure minus result not equal -100100200032");

    c = a * b;
    asserter->check(c.to_str(), "20010020003200000000000");

    c = a / b;
    asserter->check(c.to_str(), "0");
    c = b / a;
    asserter->check(c.to_str(), "2");
    
    return asserter->isOk();
}

int main(int argc, char *argv[])
{
    simpletest::SimpleTestSuite suite("bignum test");
    suite.addTester(sfcr::screate(bignum_test, suite.getAsserter()));
    return suite.run();

}
