#include <iostream>
#include <sstream>
#include <string>
#include <functional>

#include "../simpletest.h"

#include "../environment.h"
#include "../base_object.h"

#include "../data_structure_interface.h"
#include "../data_structure.h"
#include "../object.h"
#include "../type.h"

#include "../unicode.h"


using namespace std;
using namespace utakata;

class String : public data::DataEntity
{
public:
    String(std::string str) : str_(new unicode::UniString(unicode::convert(str)))
        {}
    virtual ~String() {}

    virtual void* getAddress() {return reinterpret_cast<unsigned char*>(&(*str_));}
    virtual const void* getAddress() const {return reinterpret_cast<unsigned char*>(&(*str_));}

    smart_ptr<unicode::UniString> str_;
};

bool environment_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    // Schemeの「環境」として扱われるクラスのテスト。
    // data::Environmentは、ある時点での環境を表す。
    // 環境とは、ある時点での可視である束縛変数(キーワード含む)の集合
    // であり、定義された時点で可視である変数を全て含む。
    // また、環境は親となる環境からすべて繋がっており、環境の内部にはさらに
    // 別の環境を含むが、環境を利用する側からはそれが見えないようにしている。

    interpreter::Environment env;

    smart_ptr<type::Type> hash(new type::Type("hoge"));
    smart_ptr<data::DataEntity> e(new String("hoge"));
    smart_ptr<type::TypeDescripter> t(new type::TypeDescripter(*hash, e));
    smart_ptr<data::DataSpace> ds(new data::DataSpace(t));
    smart_ptr<data::Object> o(new data::Object(false, ds));

    // 単体のenvironmentに束縛を追加する。
    // environmentに束縛を追加するには、symbolを利用しなければならないが、
    // 実際の利便性としては、symbolそのものよりも、symbol->stringで取得
    // 出来る文字列からアクセス出来たほうが楽。
    // 実際には、この追加される束縛は全てGCから返されたsmart_ptrになる。
    env.append("name", o);

    // 実際にはの部分はDataCastingHelperで処理が隠蔽される。
    unicode::UniString* tmp = reinterpret_cast<unicode::UniString*>(
        env["name"]->getSpace()->getTypeDescripter().getEntity()->getAddress());
    
    asserter->check((*tmp)[0].getRawCode(), 'h');
    asserter->check(env["name"]->enableChange(), false);
    
    return asserter->isOk();
}

int main(int argc, char *argv[])
{
    simpletest::SimpleTestSuite suite("environment test");
    suite.addTester(sfcr::screate(environment_test, suite.getAsserter()));
    suite.run();
    return 0;
}
