#include <iostream>
#include <sstream>
#include <string>
#include <functional>

#include "../simpletest.h"

#include "../type.h"
#include "base_object.h"
 
#include "../data_structure_interface.h"
#include "../data_castor.h"

using namespace std;
using namespace utakata;

class Stub : public data::DataEntity
{
public:
    Stub(std::string test) : test_(test) {}
    virtual ~Stub() {}

    virtual void* getAddress() {return static_cast<void*>(&test_);}
    virtual const void* getAddress() const {return static_cast<const void*>(&test_);}
    smart_ptr<unicode::UniString> toString() {return smart_ptr<unicode::UniString>();}

private:

    std::string test_;
};

bool typehash_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{
    // schemeのオブジェクト中で利用されるデータ及び型に対するフロントエンド
    // のテスト。

    type::Type t1("string test");
    type::Type t2("string test");

    // 同一となるはず。
    asserter->check(t1 == t2, true);
    asserter->check(t1 != t2, false);

    type::Type t3("string test1");
    asserter->check(t3 != t2, true);
    asserter->check(t3 == t2, false);

    // ハッシュのベースとなっている文字列は、テンプレート指定しているため、
    // 同一オブジェクトなら必ず一致する。
    asserter->check(t1.getHash(), t2.getHash(), "boolean ");

    type::Type t4("number test");
    // ベース文字列が違えば完全に異なる。
    asserter->check(t3 != t4, true);
    asserter->check(t3 == t4, false);
    asserter->check(t4 != t2, true);
    asserter->check(t4 == t2, false);
    
    return asserter->isOk();
}

bool typedescripter_test(smart_ptr<simpletest::SimpleTestAsserter> asserter)
{

    // 型に対する情報が設定されたデータに対するテスト。

    std::string t = "test string";
    smart_ptr<data::DataEntity> s(new Stub(std::string(t)));
    type::Type h("string");

    type::TypeDescripter des(h, s);

    // 型情報にはTypeへの直接アクセスが可能。
    asserter->check(h.getHash(), des.getType().getHash());

    // データへの直接アクセスを可能とする。
    smart_ptr<data::DataEntity> s2 = des.getEntity();

    return asserter->isOk();
}

int main(int argc, char *argv[])
{
    simpletest::SimpleTestSuite suite("environment test");
    suite.addTester(sfcr::screate(typehash_test, suite.getAsserter()));
    suite.addTester(sfcr::screate(typedescripter_test, suite.getAsserter()));
    suite.run();
    return 0;
}
