#include "type.h"
#include "data_structure.h"
#include "class.h"
#include "object.h"


using namespace utakata::type;
using namespace utakata;

type::Type::Type(const std::string& str) : hash_(0), hash_string_(str)
{
    makeHash();
}

type::Type::Type(const type::Type& t)
{
    hash_ = t.hash_;
    hash_string_ = t.hash_string_;
}

type::Type& type::Type::operator=(const type::Type& p)
{
    hash_ = p.hash_;
    hash_string_ = p.hash_string_;
    return *this;
}

void type::Type::makeHash()
{
    // 簡単にハッシュを作成する。
    std::string::const_iterator beg = hash_string_.begin(), end = hash_string_.end();

    unsigned char ch = *beg;
    for (; beg != end; ++beg)
    {
        // 各文字毎に簡単なハッシュ値を算出する。
        hash_ += (ch + (!(ch << 4)));
        ch = *beg;
    }
}

TypeDescripter::TypeDescripter(const Type& type,
                               const smart_ptr<data::DataEntity>& entity) :
    type_(type), entity_(entity)
{}

const Type& TypeDescripter::getType() const
{
    // 実体を返すが、単純に参照を返すのみとする。
    return type_;
}

smart_ptr<data::DataEntity> TypeDescripter::getEntity()
{
    return entity_;
}
