#ifndef _TYPE_H_
#define _TYPE_H_

#include <vector>
#include <string>
#include "smart_ptr.h"
#include "class.h"

namespace utakata {

    namespace data {
        class DataEntity;
        class Object;
    };

    namespace interpreter {
        class IClass;
    };

    namespace type {

        class Type
        {
            // Typeインターフェースを継承し、内部を実装した唯一のオブジェクト。
            // 全ての型は、これのテンプレートを定義して、各自のClassオブジェクトを
            // 定義する必要がある。
        public:
            Type(const std::string& name);

            Type(const Type& t);
            virtual ~Type(){}

            Type& operator=(const Type& t);

            // ハッシュ値が等しいかどうかを検査する。
            bool operator==(const Type& rh) const {return hash_ == rh.hash_;}
            bool operator!=(const Type& rh) const {return !(*this == rh);}

            const unsigned long getHash() const {return hash_;}

        private:

            void makeHash();

            // 生成されたハッシュ文字列。
            unsigned long hash_;
            std::string hash_string_;

        };

        class TypeDescripter
        {
            // 型について、次の情報を保持する。
            // ・型に対する一意なHash
            // ・実体への直接アクセス
            // ・変更可能オブジェクトかどうか。
            // Typeは、実際の作成時に設定されるのみとする。
            // TypeDescripterから取得できるDataは、あくまでデータ実体の先頭アドレス
            // を返すだけに留まり、以降は各型に対応したDataCastorを利用して、実体を
            // 取得することになる。
            // データの実体は、更に内部にあるDataEntityによって管理されるが、
            // この部分は抽象部分となっており、GCから返されるだけとなっている。
        public:
            TypeDescripter(const Type& type,
                           const smart_ptr<data::DataEntity>& entity);
            virtual ~TypeDescripter(){}

            const Type& getType() const;

            // データの実体を表すポインタを返す。
            // キャストすることで強制的にデータへのポインタにすること
            smart_ptr<data::DataEntity> getEntity();

        private:

            const Type& type_;
            smart_ptr<data::DataEntity> entity_;
        };

    };

};

#endif /* _TYPE_H_ */
